﻿using System;
using System.Collections.Generic;
using IdeaBlade.EntityModel;

namespace IdeaBlade.Extensions
{
    /// NO LONGER NECESSARY AS OF 6.0.9 BECAUSE
    /// START AND STARTPARALLEL OVERLOADS TAKE FUNC LIST
    public static class CoroutineExtensions
    {
        /// <summary>
        /// Convert a list of async functions that return <see cref="INotifyCompleted"/>
        /// into an iterator of INotifyCompleted objects.
        /// </summary>
        /// <remarks>
        /// Useful in the "function list" approach to Coroutines.
        /// See <see cref="Start"/> and <see cref="StartParallel"/>
        /// </remarks>
        public static IEnumerable<INotifyCompleted> ToIterator(this IEnumerable<Func<INotifyCompleted>> asyncFns)
        {
            var enumerator = asyncFns.GetEnumerator();
            while (enumerator.MoveNext())
                yield return enumerator.Current();
        }

        /// <summary>
        /// Executes a list of CoroutineCommands sequentially. 
        /// Waiting for each command to finish before executing the next command.
        /// </summary>

        public static CoroutineOperation Start(
                this IEnumerable<Func<INotifyCompleted>> asyncFns, Action<CoroutineOperation> completedHandler = null)
        {
            return Coroutine.Start(() => ToIterator(asyncFns), completedHandler);
        }

        /// <summary>Executes a list of CoroutineCommands in parallel.</summary>
        public static CoroutineOperation StartParallel(
           this IEnumerable<Func<INotifyCompleted>> asyncFns, Action<CoroutineOperation> completedHandler = null)
        {
            return Coroutine.StartParallel(() => ToIterator(asyncFns), completedHandler);
        }
    }

}
