﻿using System;
using System.Collections;
using System.Collections.Generic;

// ReSharper disable InconsistentNaming

namespace IdeaBlade.EntityModel {

  public class EntityQueryCoroutineOperation
    : CoroutineOperationWrapper<EntityQueryCoroutineCompletedEventArgs> {

    public EntityQueryCoroutineOperation(CoroutineOperation triggerOperation,
       Action<EntityQueryCoroutineOperation> callback = null)
      : base(new EntityQueryCoroutineCompletedEventArgs(triggerOperation)) {
      if (null == callback) return;
      Completed += delegate { callback(this); };
    }

    public IList<object> ChangedEntities {
      get { return CompletedEventArgs.ChangedEntities; }
    }

    public IEntityQuery EntityQuery {
      get { return CompletedEventArgs.EntityQuery; }
    }

    public FetchStrategy ResolvedFetchStrategy {
      get { return CompletedEventArgs.ResolvedFetchStrategy; }
    }

    public IEnumerable Results {
      get { return CompletedEventArgs.Results; }
    }

    public bool WasFetched {
      get { return CompletedEventArgs.WasFetched; }
    }
  }

  public class EntityQueryCoroutineCompletedEventArgs
    : CoroutineCompletedEventArgsWrapper<EntityQueryOperation> {

    internal EntityQueryCoroutineCompletedEventArgs(CoroutineOperation triggerOperation)
      : base(triggerOperation) {
      TargetOperationName = "EntityQueryOperation";
    }

    private EntityQueriedEventArgs QueriedEventArgs {
      get { return (EntityQueriedEventArgs) TargetEventArgs; }
    }


    public IList<object> ChangedEntities {
      get {
        EnsureCompletedSuccessfully("ChangedEntities");
        return QueriedEventArgs.ChangedEntities;
      }
    }

    public IEntityQuery EntityQuery {
      get {
        EnsureCompletedSuccessfully("Query");
        return QueriedEventArgs.Query;
      }
    }

    public FetchStrategy ResolvedFetchStrategy {
      get {
        EnsureCompletedSuccessfully("ResolvedFetchStrategy");
        return QueriedEventArgs.ResolvedFetchStrategy;
      }
    }

    public IEnumerable Results {
      get {
        EnsureCompletedSuccessfully("Results");
        return QueriedEventArgs.Results;
      }
    }

    public bool WasFetched {
      get {
        EnsureCompletedSuccessfully("WasFetched");
        return QueriedEventArgs.WasFetched;
      }
    }

  }

//------------------------------------------------------------------------------

  public class EntityQueryCoroutineOperation<T> 
    : CoroutineOperationWrapper<EntityQueryCoroutineCompletedEventArgs<T>> {

    public EntityQueryCoroutineOperation(CoroutineOperation triggerOperation,
       Action<EntityQueryCoroutineOperation<T>> callback = null)
      : base(new EntityQueryCoroutineCompletedEventArgs<T>(triggerOperation)) {
      if (null == callback) return;
      Completed += delegate { callback(this); };
    }

    public IList<object> ChangedEntities {
      get { return CompletedEventArgs.ChangedEntities; }
    }

    public IEntityQuery<T> EntityQuery {
      get { return CompletedEventArgs.EntityQuery; }
    }

    public FetchStrategy ResolvedFetchStrategy {
      get { return CompletedEventArgs.ResolvedFetchStrategy; }
    }

    public IEnumerable<T> Results {
      get { return CompletedEventArgs.Results; }
    }

    public bool WasFetched {
      get { return CompletedEventArgs.WasFetched; }
    }
  }

  public class EntityQueryCoroutineCompletedEventArgs<T>
    : CoroutineCompletedEventArgsWrapper<EntityQueryOperation<T>> {

    internal EntityQueryCoroutineCompletedEventArgs(CoroutineOperation triggerOperation)
      : base(triggerOperation) {
      TargetOperationName = String.Format("EntityQueryOperation<{0}>", typeof(T).Name);
    }

    private EntityQueriedEventArgs<T> QueriedEventArgs {
      get { return (EntityQueriedEventArgs<T>) TargetEventArgs; }
    }


    public IList<object> ChangedEntities {
      get {
        EnsureCompletedSuccessfully("ChangedEntities");
        return QueriedEventArgs.ChangedEntities;
      }
    }

    public IEntityQuery<T> EntityQuery {
      get {
        EnsureCompletedSuccessfully("Query");
        return QueriedEventArgs.Query;
      }
    }

    public FetchStrategy ResolvedFetchStrategy {
      get {
        EnsureCompletedSuccessfully("ResolvedFetchStrategy");
        return QueriedEventArgs.ResolvedFetchStrategy;
      }
    }

    public IEnumerable<T> Results {
      get {
        EnsureCompletedSuccessfully("Results");
        return QueriedEventArgs.Results;
      }
    }

    public bool WasFetched {
      get {
        EnsureCompletedSuccessfully("WasFetched");
        return QueriedEventArgs.WasFetched;
      }
    }

  }

}