﻿
namespace IdeaBlade.EntityModel {

  /// <summary>
  /// Supplements native <see cref="EntityStates"/> with some
  /// useful combinations. May become part of DevForce someday
  /// </summary>
  public static class EntityStates {
  
    /// <summary>
    /// Synthetic EntityState, the bit-wise OR of all states that can appear in entity cache.
    /// </summary>
    public const EntityState AllCachedStates =
      EntityState.Added | EntityState.Deleted |
      EntityState.Modified | EntityState.Unchanged;

    /// <summary>
    /// Synthetic EntityState, the bit-wise OR of all possible states.
    /// </summary>
    public const EntityState AllStates = AllCachedStates | EntityState.Detached;

    /// <summary>
    /// The entity has been added to the EntityManager.
    /// When saved, the entity will be inserted into the database
    /// </summary>
    public const EntityState Added = EntityState.Added;

    /// <summary>
    /// The entity was marked for deletion using the Delete() method.
    /// When saved, the entity will be deleted from the database.
    /// </summary>
    public const EntityState Deleted = EntityState.Deleted;

    /// <summary>
    /// The entity does not belong to an EntityManager.
    /// It may have been created but not added to the EntityManager
    /// or it may have been removed from an EntityManager.
    /// </summary>
    public const EntityState Detached = EntityState.Detached;

    /// <summary>
    /// The entity has been modified.
    /// When saved, the database will be updated with the changed entity.
    /// </summary>
    public const EntityState Modified = EntityState.Modified;

    /// <summary>
    /// The entity has not changed since it was last queried or saved.
    /// </summary>
    public const EntityState Unchanged = EntityState.Unchanged;

  }
}