using System;

namespace IdeaBlade.Extensions {

  /// <summary>
  /// Replacement for <see cref="DateTime"/>
  /// ** ALWAYS USE THIS INSTEAD OF CALLING DATETIME DIRECTLY
  /// </summary>
  /// <remarks>
  /// Covers the <see cref="DateTime"/> static methods so that 
  /// test and development scenarios can reset the "Current" time
  /// to whatever they like. See <see cref="NowFn"/>.
  /// TODO: Extend to support UtcNow
  /// </remarks>
  public static class SystemTime {

    /// <summary>
    /// Gets a <see cref="DateTime"/> that is set to the "current" date and time
    /// in the manner of <see cref="DateTime.Now"/>.
    /// See <see cref="NowFn"/>.
    /// </summary>
    public static DateTime Now() { return NowFn(); }

    /// <summary>
    /// Gets "current" date
    /// in the manner of <see cref="DateTime.Today"/>.
    /// See <see cref="NowFn"/>.
    /// </summary>
    public static DateTime Today() {
      var now = Now();
      return now-now.TimeOfDay;
    }

    /// <summary>
    /// Function for getting the "current" date and time.
    /// Defaults to <see cref="DateTime.Now"/> but
    /// could be replaced in a test or development scenario.
    /// </summary>
    public static Func<DateTime> NowFn = () => DateTime.Now;

    /// <summary>
    /// Reset <see cref="NowFn"/> to application default
    /// </summary>
    public static void ResetNowFn() { NowFn  = () => DateTime.Now; }
  }
}