Namespace IdeaBlade.Extensions

  ''' <summary>
  ''' Replacement for <see cref="DateTime"/>
  ''' ** ALWAYS USE THIS INSTEAD OF CALLING DATETIME DIRECTLY
  ''' </summary>
  ''' <remarks>
  ''' Covers the <see cref="DateTime"/> static methods so that 
  ''' test and development scenarios can reset the "Current" time
  ''' to whatever they like. See <see cref="NowFn"/>.
  ''' TODO: Extend to support UtcNow
  ''' </remarks>
  Public NotInheritable Class SystemTime

	''' <summary>
	''' Gets a <see cref="DateTime"/> that is set to the "current" date and time
	''' in the manner of <see cref="DateTime.Now"/>.
	''' See <see cref="NowFn"/>.
	''' </summary>
	Private Sub New()
	End Sub
	Public Shared Function Now() As Date
		Return NowFn()
	End Function

	''' <summary>
	''' Gets "current" date
	''' in the manner of <see cref="DateTime.Today"/>.
	''' See <see cref="NowFn"/>.
	''' </summary>
	Public Shared Function Today() As Date
	  Dim now = SystemTime.Now()
	  Return now-now.TimeOfDay
	End Function

	''' <summary>
	''' Function for getting the "current" date and time.
	''' Defaults to <see cref="DateTime.Now"/> but
	''' could be replaced in a test or development scenario.
	''' </summary>
	Public Shared NowFn As Func(Of Date) = Function() Date.Now

	''' <summary>
	''' Reset <see cref="NowFn"/> to application default
	''' </summary>
	Public Shared Sub ResetNowFn()
		NowFn = Function() Date.Now
	End Sub
  End Class
End Namespace