Imports IdeaBlade.TestFramework
Imports Microsoft.Silverlight.Testing
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports SoftwareApproach.TestingExtensions

Imports IdeaBlade.EntityModel
Imports SimpleNorthwindModel

' ReSharper disable InconsistentNaming

Namespace Querying.AsyncQueryInclude

  <TestClass>
  Public Class When_query_with_include
	  Inherits DevForceTest

	<TestMethod, Asynchronous, Timeout(15000), Tag("Include")>
	Public Sub then_can_get_Orders_with_Customers()

		   ' Assertion: the query retrieved orders with the customers
		   '   "ChangedEntities" reports entities from this particular query 
		   '    that changed the EntityManager cache
		   '    Contains any retrieved entity which was added to the cache or which
		   '    modified an entity already in the cache.
		   '    If we ran the query again for this manager,
		   '    "ChangedEntities would be empty.
      DoItAsync(Sub()
                  Dim custQueryResult = New List(Of Customer)()
                  Dim manager = New NorthwindManager()
                  AddHandler manager.Customers.Take(5).Include("Orders").ExecuteAsync().Completed, Sub(sender, args)
                                                                                                     If args.HasError Then
                                                                                                       Assert.Fail("Customers query error: " & args.Error.ToString())
                                                                                                     End If
                                                                                                     custQueryResult.AddRange(args.Results)
                                                                                                     args.ChangedEntities.Any(Function(entity) TypeOf entity Is Order).ShouldBeTrue("Did not get orders with my customers")
                                                                                                     TestComplete()
                                                                                                   End Sub
                End Sub)
	End Sub

	<TestMethod, Asynchronous, Timeout(15000), Tag("Include")>
	Public Sub then_can_get_Orders_with_Customers_with_no_magic_string()

      DoItAsync(Sub()
                  Dim custQueryResult = New List(Of Customer)()
                  Dim manager = New NorthwindManager()
                  AddHandler manager.Customers.Take(5).Include(Function(c) c.Orders).ExecuteAsync().Completed, Sub(sender, args)
                                                                                                                 If args.HasError Then
                                                                                                                   Assert.Fail("Customers query error: " & args.Error.ToString())
                                                                                                                 End If
                                                                                                                 custQueryResult.AddRange(args.Results)
                                                                                                                 args.ChangedEntities.Any(Function(entity) TypeOf entity Is Order).ShouldBeTrue("Did not get orders with my customers")
                                                                                                                 TestComplete()
                                                                                                               End Sub
                End Sub)
	End Sub

	' Verify that it doesn't go to the server during navigation
	' by preventing the manager from trying
	 <TestMethod, Asynchronous, Timeout(15000), Tag("Include")>
    Public Sub then_does_not_go_to_server_for_Customer_Orders()
      DoItAsync(Sub()
                  Dim custQueryResult = New List(Of Customer)()
                  Dim manager = New NorthwindManager()
                  Dim op = manager.Customers.Take(5).Include(Function(c) c.Orders).ExecuteAsync()
                  AddHandler op.Completed, (Sub(sender, args)
                                              If (args.HasError) Then
                                                Assert.Fail("Customers query error: " & args.Error.ToString())
                                              End If
                                              custQueryResult.AddRange(args.Results)
                                              manager.Disconnect()
                                              AddHandler manager.Fetching, Sub() Assert.Fail("Tried to navigate to the server")
                                              Dim firstCustomer As Customer = custQueryResult.First()
                                              Dim firstorders As IList(Of Order) = firstCustomer.Orders
                                              firstorders.ShouldNotBeEmpty("Did not get orders with my customers")
                                              TestComplete()
                                            End Sub)
                End Sub)
    End Sub

    <TestMethod(), Asynchronous(), Timeout(15000), Tag("Include")>
    Public Sub then_can_get_Orders_and_OrderDetails_of_Customers()

      ' Can't use strong typing (lambda) if a middle leg
      ' is a collection. Here "Orders" is a collection
      DoItAsync(Sub()
                  Dim custQueryResult = New List(Of Customer)()
                  Dim manager = New NorthwindManager()
                  AddHandler manager.Customers.Take(5).Include("Orders.OrderDetails").ExecuteAsync().Completed, Sub(sender, args)
                                                                                                                  If args.HasError Then
                                                                                                                    Assert.Fail("Customers query error: " & args.Error.ToString())
                                                                                                                  End If
                                                                                                                  custQueryResult.AddRange(args.Results)
                                                                                                                  args.ChangedEntities.Any(Function(entity) TypeOf entity Is Order).ShouldBeTrue("Did not get Orders with my customers")
                                                                                                                  args.ChangedEntities.Any(Function(entity) TypeOf entity Is OrderDetail).ShouldBeTrue("Did not get OrderDetails with my customers")
                                                                                                                  TestComplete()
                                                                                                                End Sub
                End Sub)
    End Sub
  End Class
  '---------------------------------------------------

  ''' <summary>
  ''' Explore asynchronous queries without "Include"
  ''' Use entity navigation to determine if entities are in cache.
  ''' an approach that requires preventing "Lazy load" by
  ''' blocking the EntityManager from querying the database inside
  ''' a navigation property.
  ''' </summary>
  <TestClass()>
  Public Class When_async_query_without_include

    Inherits AsyncQueryIncludeContext

    ''' <remarks>
    ''' Use entity navigation to determine if entities are in cache.
    ''' an approach that requires preventing "Lazy load" by
    ''' blocking the EntityManager from querying the database inside
    ''' a navigation property.
    ''' </remarks>
    <TestMethod(), Asynchronous(), Timeout(20000), Tag("Integration"), Tag("QueryInclude")>
    Public Sub then_dont_get_Nancys_Orders_from_cache_by_navigation()

      ' Use FirstOrNullEntity to avoid possible NullReferenceException during navigation
      ' Should get an empty list after navigating from Nancy to her Orders 
      DoItAsync(Function() NancyAsyncQuery.FirstOrNullEntity().TestTheResult(Sub(nancy)
                                                                               ForceManagerLocal()
                                                                               nancy.Orders.ShouldBeEmpty("Should NOT find orders for Nancy")
                                                                             End Sub))
    End Sub

    ''' <remarks>
    ''' Use <see cref="EntityManager.FindEntities{T}(EntityState)"/>
    ''' to determine if entities are in cache.
    ''' </remarks>
    <TestMethod(), Asynchronous(), Timeout(20000), Tag("Integration"), Tag("QueryInclude")>
    Public Sub then_Nancy_does_not_have_orders_in_cache()

      ' We don't care about the query result so FirstOrDefault is fine
      ' Use FindEntities to poke around in the EntityManager's entity cache 
      DoItAsync(Function() NancyAsyncQuery.FirstOrDefault().TestTheResult(Sub(result)
                                                                            Dim orders = Manager.FindEntities(Of Order)(EntityStates.AllCachedStates)
                                                                            orders.ShouldBeEmpty("Should NOT find orders for Nancy")
                                                                          End Sub))
    End Sub

  End Class

  '-------------------------------------------------------------------------

  ''' <summary>
  ''' Explore asynchronous queries with and without "Include"
  ''' Use entity navigation to determine if entities are in cache.
  ''' an approach that requires preventing "Lazy load" by
  ''' blocking the EntityManager from querying the database inside
  ''' a navigation property.
  ''' </summary>
  <TestClass()>
  Public Class When_async_query_with_include
    Inherits AsyncQueryIncludeContext

    ''' <remarks>
    ''' Use entity navigation to determine if entities are in cache.
    ''' an approach that requires preventing "Lazy load" by
    ''' blocking the EntityManager from querying the database inside
    ''' a navigation property.
    ''' </remarks>
    <TestMethod(), Asynchronous(), Timeout(20000), Tag("Integration"), Tag("QueryInclude")>
    Public Sub then_prove_can_get_Nancys_Orders_from_cache_locally()

      ' Use FirstOrNullEntity to avoid possible NullReferenceException during navigation
      ' Should get a full list after navigating from Nancy to her Orders 
      DoItAsync(Function() NancyAsyncQueryWithInclude.FirstOrNullEntity().TestTheResult(Sub(nancy)
                                                                                          ForceManagerLocal()
                                                                                          nancy.Orders.ShouldNotBeEmpty("Should have orders with Nancy's ID")
                                                                                        End Sub))
    End Sub

    ''' <remarks>
    ''' Use <see cref="EntityManager.FindEntities{T}(EntityState)"/>
    ''' to determine if entities are in cache.
    ''' </remarks>
    <TestMethod(), Asynchronous(), Timeout(20000), Tag("Integration"), Tag("QueryInclude")>
    Public Sub then_Nancy_has_orders_in_cache()
      ' We don't care about the query result so FirstOrDefault is fine
      ' Use FindEntities to poke around in the EntityManager's entity cache 
      DoItAsync(Function() NancyAsyncQueryWithInclude.FirstOrDefault().TestTheResult(Sub(nancy)
                                                                                       Dim orders = Manager.FindEntities(Of Order)(EntityStates.AllCachedStates)
                                                                                       orders.ShouldNotBeEmpty("Should have orders with Nancy's ID")
                                                                                     End Sub))
    End Sub
  End Class

  '-------------------------------------------------------------------------

  Public Class AsyncQueryIncludeContext
    Inherits DevForceTest

    Protected Overrides Sub Context()
      ' Create manager that connects to the server automatically
      Manager = New NorthwindManager()
      PinManagerToSyncContextThread(Manager)
    End Sub
#Region "Helpers"

    ''' <summary>Get and set the EntityManager used by test methods.</summary>
    Protected Property Manager() As NorthwindManager

    Protected Sub ForceManagerLocal()
      Manager.ForceManagerLocal()
    End Sub

    ''' <summary>Employee Id known to be the ID of "Nancy Davolio"</summary>
    Protected Const NancyDavolioId As Integer = 1

    ''' <summary>
    ''' Get a Scalar async query regarding the employee with Nancy's Id.
    ''' </summary>
    Protected ReadOnly Property NancyAsyncQuery() As IEntityScalarQuery(Of Employee)
      Get
        ' Which LINQ style do you prefer?

        ' Fluent syntax
        Return Manager.Employees.Where(Function(e) e.EmployeeID = NancyDavolioId).AsScalarAsync()

        '        
        '              // Query syntax
        '              var q = from emp in Manager.Employees
        '                      where emp.EmployeeID == NancyDavolioId
        '                      select emp;
        '              return q.AsScalarAsync();
        '        
      End Get

    End Property

    ''' <summary>
    ''' Get a Scalar async query regarding the employee with Nancy's Id that
    ''' has an "Include" statement which 
    ''' brings the related orders and order details along for the ride.
    ''' </summary>
    Protected ReadOnly Property NancyAsyncQueryWithInclude() As IEntityScalarQuery(Of Employee)

      Get
        ' Which LINQ style do you prefer?

        '        
        '              // Fluent syntax
        '              return Manager.Employees
        '                .Where(e => e.EmployeeID == NancyDavolioId)
        '                .Include("Orders.OrderDetails") // The "Include" makes the difference
        '                .AsScalarAsync();
        '        

        ' Query syntax
        Dim q = From emp In Manager.Employees
                Where emp.EmployeeID = NancyDavolioId
                Select emp
        q = q.Include("Orders.OrderDetails") ' The "Include" makes the difference
        Return q.AsScalarAsync()
      End Get
    End Property

#End Region

  End Class

End Namespace
