﻿Imports IdeaBlade.Core
Imports Microsoft.Silverlight.Testing
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports SoftwareApproach.TestingExtensions ' for Silverlight

Imports IdeaBlade.EntityModel
Imports IdeaBlade.TestFramework
Imports SimpleNorthwindModel

' ReSharper disable InconsistentNaming
' ReSharper disable UseObjectOrCollectionInitializer

Namespace Querying.CoroutineTests

	<TestClass>
	Public Class When_using_function_lists
		Inherits DevForceTest

		''' <summary>Executed before every test method.</summary>
		Protected Overrides Sub Context()
			' Create manager that connects to the server automatically
			Manager = New NorthwindManager()
			PinManagerToSyncContextThread(Manager)

		End Sub

		Private Property Manager() As NorthwindManager

		' Just prove we can access the database
		<TestMethod, Asynchronous, Timeout(20000), Tag("Integration"), Tag("Coroutine")>
		Public Sub can_get_customers()
			DoItAsync(Function() Manager.Customers.ExecuteAsync().TestTheResults(Function(custs) custs.ShouldNotBeEmpty("Should have customers"), "Customers query"))
		End Sub

		<TestMethod, Asynchronous, Timeout(20000), Tag("Integration"), Tag("Coroutine")>
		Public Sub can_update_customers_with_serial_fnc_list()
			DoItAsync(Sub()
				Dim funcList = MakeSerialAsyncFunctionList()
				Coroutine.Start(funcList, Sub(op)
					If op.CompletedSuccessfully Then
						TestComplete()
					Else
						op.MarkErrorAsHandled()
						Throw op.Error
					End If
				End Sub)
			End Sub)
		End Sub

		''' <summary>
		''' Make list of asynchronous functions to be executed serially
		''' </summary>
		Private Function MakeSerialAsyncFunctionList() As IEnumerable(Of Func(Of INotifyCompleted))
			Dim funcList = New List(Of Func(Of INotifyCompleted))()

			' query for customers with non-null Countries
			Dim custQuery = Manager.Customers.Where(Function(c) c.Country IsNot Nothing)

			funcList.Add(Function() custQuery.ExecuteAsync())

			' Make a change to them and save them
			funcList.Add(Function()
				UpdateCachedCustomers()
				Return Manager.SaveChangesAsync()
			End Function)

			' Clear, re-load
			funcList.Add(Function()
				Manager.Clear()
				Return custQuery.ExecuteAsync()
			End Function)

			' Verify change then restore customers to original state
			funcList.Add(Function()
				RestoreCachedCustomer()
				Return Manager.SaveChangesAsync()
			End Function)

			Return funcList
		End Function

		<TestMethod, Asynchronous, Timeout(20000), Tag("Integration"), Tag("Coroutine")>
		Public Sub can_get_many_types_with_parallel_fnc_list()
			DoItAsync(Sub()
				Manager.GetEntityGroups().ShouldHaveCountOf(0, "Manager is not empty.")
				Dim funcList = New List(Of Func(Of INotifyCompleted)) From {Function() Manager.Products.ExecuteAsync(), Function() Manager.Employees.ExecuteAsync(), Function() Manager.Customers.ExecuteAsync()}
				Coroutine.StartParallel (funcList, Sub(op)
					If op.CompletedSuccessfully Then
						Manager.GetEntityGroups().Select(Function(g) g.EntityType).ShouldContainAllOf( { GetType(Product), GetType(Employee), GetType(Customer)}, "Manager doesn't have expected entity types.")
						TestComplete()
					Else
						op.MarkErrorAsHandled()
						Throw op.Error
					End If
				End Sub)
			End Sub)
		End Sub

		#Region "Helpers"

		Private Const _testSuffix As String = "###"

		Private Sub UpdateCachedCustomers()
      Manager.FindEntities(Of Customer)(EntityState.Unchanged).ForEach(Sub(c) c.Address = c.Address & _testSuffix)
		End Sub

		Private Sub RestoreCachedCustomer()
							 ' Fail if any address is missing the test suffix
      Manager.FindEntities(Of Customer)(EntityState.Unchanged).ForEach(Sub(c)
                                                                         If Nothing IsNot c.Address Then
                                                                           Dim addr = c.Address
                                                                           Assert.IsTrue(addr.Contains(_testSuffix), c.CompanyName & " address is missing text suffix.")
                                                                           c.Address = c.Address.Substring(0, c.Address.IndexOf(_testSuffix))
                                                                         End If
                                                                       End Sub)
		End Sub

		#End Region

	End Class
End Namespace