Imports Microsoft.Silverlight.Testing
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports SoftwareApproach.TestingExtensions

Imports IdeaBlade.EntityModel
Imports IdeaBlade.TestFramework
Imports SimpleNorthwindModel

' ReSharper disable InconsistentNaming

Namespace Querying.EasyQuery
  <TestClass>
  Public Class When_trying_simple_asynchronous_queries
	  Inherits DevForceTest

	''' <summary>Executed before every test method.</summary>
	Protected Overrides Sub Context()
	  ' Create manager that connects to the server automatically
	  Manager = New NorthwindManager()
	  PinManagerToSyncContextThread(Manager)
	End Sub

	Private Property Manager() As NorthwindManager
	Private Const NancyDavolioId As Integer = 1

	<TestMethod, Asynchronous, Timeout(20000), Tag("Integration"), Tag("EasyQuery")>
	Public Sub then_get_customers_with_Model_EntityManager()

	  DoItAsync(Function() Manager.Customers.ExecuteAsync().TestTheResults(Function(custs) custs.ShouldNotBeEmpty("Should have customers"), "Customers query")) ' "Get all customers" query
	End Sub

	<TestMethod, Asynchronous, Timeout(20000), Tag("Integration"), Tag("EasyQuery")>
	Public Sub then_get_customers_with_DevForce_EntityManager()
		' DevForce EntityManager knows nothing about the model
      DoItAsync(Sub()
                  Dim dfManager = New EntityManager()
                  dfManager.GetQuery(Of Customer)().ExecuteAsync().TestTheResults(Function(custs) custs.ShouldNotBeEmpty("Should have customers"), "Customers query")
                End Sub)
    End Sub

    <TestMethod(), Asynchronous(), Timeout(20000)>
    <Tag("Integration"), Tag("EasyQuery")>
    Public Sub then_get_customers_verbose()

      ' Same query but with "TestTheResults" inline
      DoItAsync(Sub()
                  Manager.GetQuery(Of Customer)().ExecuteAsync().TestTheResults(Function(custs) custs.ShouldNotBeEmpty("Should have customers"), "Customers query")
                  '
                  '  Could not get VB version below to work
                  '
                  '  Manager.Customers.ExecuteAsync().OnCancelled(Sub() Assert.Fail("Unexpected cancellation of Customers query")).OnError(Sub(err)
                  '                                                                                                                          Assert.Fail("Customers query failed with exception: " & err.ToString())
                  '                                                                                                                        End Sub).OnSuccess(Sub(custs) custs.ShouldNotBeEmpty("Should have customers")).OnCompleted(Sub(args)
                  '                                                                                                                                                                                                                     WaitFor.TestComplete()
                  '                                                                                                                                                                                                                   End Sub)
                  'End Sub)
                End Sub)
    End Sub

    <TestMethod(), Asynchronous(), Timeout(20000), Tag("Integration"), Tag("EasyQuery")>
    Public Sub then_get_Nancy_Davolio_with_fluent_syntax()

      ' Fluent syntax for query for employees with ID == to Nancy's ID
      ' Only want one (the first)
      ' Turn into "Scalar" query and ask for First (or null if none)
      ' "queryOp" is an EntityQueryOperation<Employee>
      ' TestTheResult is a DevForce Operation Extension for testing
      ' Put your tests inside its parameters
      DoItAsync(Sub()
                  Dim nancyQuery = Manager.Employees.Where(Function(e) e.EmployeeID = NancyDavolioId)
                  Dim queryOp = nancyQuery.AsScalarAsync().FirstOrDefault().TestTheResult(Sub(nancy)
                                                                                            nancy.ShouldNotBeNullOrNullEntity("Can't find Nancy?")
                                                                                            nancy.LastName.ShouldEqual("Davolio")
                                                                                          End Sub)
                End Sub)
    End Sub


    <TestMethod(), Asynchronous(), Timeout(20000), Tag("Integration"), Tag("EasyQuery")>
    Public Sub then_get_Nancy_Davolio_with_query_syntax()

      ' "Query Expression" or "Comprehension" syntax
      ' Query for employees with ID == to Nancy's ID
      ' Test the "Scalar" query that asks for First (or null if none)
      ' Verbose instead of TestTheResult
      DoItAsync(Sub()
                  Dim nancyQuery = From emp In Manager.Employees
                                   Where emp.EmployeeID = NancyDavolioId
                                   Select emp
                  nancyQuery.AsScalarAsync().FirstOrDefault().OnCancelled(Sub() Assert.Fail("Unexpected cancellation")).OnError(Sub(err) Assert.Fail("Query failed with exception: " & err.ToString())).OnSuccess(Sub(nancy)
                                                                                                                                                                                                                    nancy.EntityAspect.IsNullEntity.ShouldBeFalse("Can't find Nancy?")
                                                                                                                                                                                                                    nancy.LastName.ShouldEqual("Davolio")
                                                                                                                                                                                                                  End Sub).OnCompleted(Sub() TestComplete())
                End Sub)
    End Sub


    <TestMethod(), Asynchronous(), Timeout(20000), Tag("Integration"), Tag("EasyQuery")>
    Public Sub then_get_Nancy_Davolio_with_FirstOrNullEntity()
      ' Fluent syntax
      ' Verbose instead of TestTheResult
      DoItAsync(Sub()
                  Dim nancyQuery = Manager.Employees.Where(Function(e) e.EmployeeID = NancyDavolioId)
                  nancyQuery.AsScalarAsync().FirstOrNullEntity().OnCancelled(Sub() Assert.Fail("Unexpected cancellation")).OnError(Sub(err) Assert.Fail("Query failed with exception: " & err.ToString())).OnSuccess(Sub(nancy)
                                                                                                                                                                                                                       nancy.EntityAspect.IsNullEntity.ShouldBeFalse("Can't find Nancy?")
                                                                                                                                                                                                                       nancy.LastName.ShouldEqual("Davolio")
                                                                                                                                                                                                                     End Sub).OnCompleted(Sub() TestComplete())
                End Sub)

    End Sub

    <TestMethod(), Asynchronous(), Timeout(20000), Tag("Integration"), Tag("EasyQuery")>
    Public Sub then_get_Nancys_orders_using_Callback()
      DoItAsync(Sub() Manager.Orders.Where(Function(o) o.EmployeeID = NancyDavolioId).ExecuteAsync(Sub(op)
                                                                                                     If (op.HasError) Then
                                                                                                       op.HasError.ShouldBeFalse("Query failed with exception: " & op.Error.ToString())
                                                                                                     Else
                                                                                                       op.HasError.ShouldBeFalse("Query failed with exception: ")
                                                                                                     End If
                                                                                                     op.Results.ShouldNotBeEmpty("No orders for Nancy?")
                                                                                                     TestComplete()
                                                                                                   End Sub))

    End Sub

    <TestMethod(), Asynchronous(), Timeout(20000), Tag("Integration"), Tag("EasyQuery")>
    Public Sub then_get_Nancys_orders_via_Completed_event()

      DoItAsync(Sub() Manager.Orders.Where(Function(o) o.EmployeeID = NancyDavolioId).ExecuteAsync().OnCompleted(Sub(args)
                                                                                                                   args.HasError.ShouldBeFalse("Query failed with exception: " & args.ToString())
                                                                                                                   args.Results.ShouldNotBeEmpty("No orders for Nancy?")
                                                                                                                   TestComplete()
                                                                                                                 End Sub))

    End Sub

  End Class
End Namespace
